#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h> 
#include "binio.h"

/*********************************************************
 * java_ser: connect to server and send the data
 *     
 *
 * return value (all functions)
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 *********************************************************/





/******************************************************************
 * initialize the socket to send the data to the isosurface server
 * return socket descriptor. If failed return -1
 ******************************************************************/
int initialize2()
{
   int sockfd;

   char *hostname;
   char *rawport;
   int port;

   struct  hostent *h;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("AVSHOST");
   rawport = getenv("AVSPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ... \n");
      return -1;
   }
   
   if (rawport == NULL) 
      port = 7655;
   else
      port = atoi(rawport);

   if ((h=gethostbyname(hostname)) == NULL) {  /* get the host info */
/*    herror("gethostbyname");  */
		fprintf(stderr,"initialize2: gethostbyname(%s) failed.\n",hostname);
      return -1;
   }

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = 
                    inet_addr(inet_ntoa(*((struct in_addr *)h->h_addr)));
   serv_addr.sin_port = htons(port);

   /*
   printf("Host name  : %s\n", h->h_name);
   printf("IP Address : %s\n",inet_ntoa(*((struct in_addr *)h->h_addr)));
   */

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return -1;
   }

   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to Isosurface server.\n");
      return -1;
   }

   return sockfd;

}



/*************************************************************
 * java_ser3d_
 * c program to link with fortran 90
 * This program writes the xsize, ysize, zsize and z values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used for title
 *   time - time value
 *   n    - x dimension 
 *   x   - x array
 *   y   - y array
 *   z   - z array
 *   grid   - grid array
 ************************************************************/
int avs_rect_(name, time, n, x,y,z,grid) 
  char *name;
  double *time;
  int n[];
  double grid[],x[],y[],z[];
{
   int sockfd;
   int  name_length, i;
   int size;
   double ft = *time; 
   float ftime = (float)ft;
   int command = 1;
/*    float *tgrid; */
   size=1;
   for (i=0;i<3;i++) size*=n[i];
/*    tgrid = (float *)malloc(sizeof(float)*size);   */

   sockfd = initialize2();
   if (sockfd == -1)
     return 1;

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, 1);     /* send no of time */
   write_float4(sockfd, ftime);
   write_int4(sockfd,3); /*send the rank*/

   write_int4(sockfd, n[0]);
   write_int4(sockfd, n[1]);
   write_int4(sockfd, n[2]);

/*    for (i=0;i<n[0];i++){tgrid[i] = (float)x[i];} */
/*    write_float4a(sockfd, tgrid,n[0]); */

/*    for (i=0;i<n[1];i++){tgrid[i] = (float)y[i];} */
/*    write_float4a(sockfd, tgrid,n[1]); */

/*    for (i=0;i<n[2];i++){tgrid[i] = (float)z[i];} */
/*    write_float4a(sockfd, tgrid,n[2]); */
/*    for (i=0;i<size;i++){tgrid[i] = (float)grid[i];} */
/*    write_float4a(sockfd, tgrid,size); */
   for (i=0;i<n[0];i++) write_float4(sockfd,(float)x[i]);
   for (i=0;i<n[1];i++) write_float4(sockfd,(float)y[i]);
   for (i=0;i<n[2];i++) write_float4(sockfd,(float)z[i]);
   for (i=0;i<size;i++) write_float4(sockfd,(float)grid[i]);
/*    free(tgrid); */
   close(sockfd);
   return 0;
}

int avs_rect(char *name, double time, int n[],  double x[],double y[], double z[], double grid[]) {
   return avs_rect_(name, &time, n, x,y,z,grid);
}


int avs_struct_(name, time, n, x,y,z,grid) 
  char *name;
  double *time;
  int n[];
  double grid[],x[],y[],z[];
{
   int sockfd;
   int  name_length, i;
   int size;
   double ft = *time; 
   float ftime = (float)ft;
   int command = 2;
/*    float *tgrid; */
   size=1;
   for (i=0;i<3;i++) size*=n[i];
/*    tgrid = (float *)malloc(sizeof(float)*size);   */

 
   sockfd = initialize2();
   if (sockfd == -1)
     return 1;

   /* first send command */
   write_int4(sockfd, command);

   /* send title information */
   name_length = strlen(name);
   write_int4(sockfd, name_length);
   write(sockfd, name, name_length);

   /* write time and sizes and threshold */
   write_int4(sockfd, 1);     /* send no of time */
   write_float4(sockfd, ftime);
   write_int4(sockfd,3); /*send the rank*/

   write_int4(sockfd, n[0]);
   write_int4(sockfd, n[1]);
   write_int4(sockfd, n[2]);

/*    for (i=0;i<size;i++){tgrid[i] = (float)x[i];} */

/*    write_float4a(sockfd, tgrid,size); */

/*    for (i=0;i<size;i++){tgrid[i] = (float)y[i];} */
/*    write_float4a(sockfd, tgrid,size); */

/*    for (i=0;i<size;i++){tgrid[i] = (float)z[i];} */
/*    write_float4a(sockfd, tgrid,size); */
/*    for (i=0;i<size;i++){tgrid[i] = (float)grid[i];} */
/*    write_float4a(sockfd, tgrid,size); */
   for (i=0;i<size;i++) write_float4(sockfd,(float)x[i]);
   for (i=0;i<size;i++) write_float4(sockfd,(float)y[i]);
   for (i=0;i<size;i++) write_float4(sockfd,(float)z[i]);
   for (i=0;i<size;i++) write_float4(sockfd,(float)grid[i]);

/*    free(tgrid); */
   close(sockfd);
   return 0;
}

int avs_struct(char *name, double time, int n[],  double x[],double y[], double z[], double grid[]) {
   return avs_struct_(name, &time, n, x,y,z,grid);
}
